# -*- coding: utf-8 -*-

import os

from django.conf import settings
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _


def generate_filename(owner, file_name, prefix_filepath=''):
    owner_path = f"{owner.id}-{slugify(owner.email)}"
    return os.path.join(settings.MEDIA_PRIVATE_ROOT, prefix_filepath, owner_path, file_name)


def makedirs(full_path):
    # From django.core.files.storage.Storage._save method

    # Create any intermediate directories that do not exist.
    directory = os.path.dirname(full_path)
    try:
        if settings.FILE_UPLOAD_DIRECTORY_PERMISSIONS is not None:
            # Set the umask because os.makedirs() doesn't apply the "mode"
            # argument to intermediate-level directories.
            old_umask = os.umask(0o777 & ~settings.FILE_UPLOAD_DIRECTORY_PERMISSIONS)
            try:
                os.makedirs(directory, settings.FILE_UPLOAD_DIRECTORY_PERMISSIONS, exist_ok=True)
            finally:
                os.umask(old_umask)
        else:
            os.makedirs(directory, exist_ok=True)
    except FileExistsError:
        raise FileExistsError('%s exists and is not a directory.' % directory)


def write_private_file(owner, file_obj, prefix_filepath=''):
    """
    Upload file on private user dir
    owner: instance of User
    file_obj: instance of file object
    prefix_filepath: prefix path after MEDIA_PRIVATE_ROOT
    """
    filename = generate_filename(owner, file_obj.name, prefix_filepath)
    makedirs(filename)
    with open(filename, 'wb') as f:
        try:
            f.write(file_obj.read())
            return True, filename
        except Exception as e:
            return False, str(e)


def delete_private_file(owner, filename, prefix_filepath=''):
    """
    Delete file from private user dir
    owner: instance of User
    filename: file name and path
    prefix_filepath: prefix path after MEDIA_PRIVATE_ROOT
    """
    file_name = os.path.basename(filename)
    filename_new = generate_filename(owner, file_name, prefix_filepath)
    if filename != filename_new:
        return False, _('File not found')
    try:
        os.remove(filename)
        return True, filename
    except Exception as e:
        return False, str(e)
